'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var isBrowser = typeof window !== 'undefined' && typeof window.document !== 'undefined' && typeof window.document.createElement !== 'undefined' && !(process.env.NODE_ENV !== 'production' && process.env.USE_GLOBAL_CONTEXT_TEST_MODE === 'server');

var useIsomorphicLayoutEffect = isBrowser ? React.useLayoutEffect : function (fn) {
  return fn();
};

var error = function error(message) {
  if (process.env.NODE_ENV === 'production') {
    return;
  }
  /* eslint no-console: 0 */


  console.error(message);
};
var warn = function warn(message) {
  if (process.env.NODE_ENV === 'production') {
    return;
  }
  /* eslint no-console: 0 */


  console.warn(message);
};
var log = function log(message) {
  if (process.env.NODE_ENV === 'production') {
    return;
  }
  /* eslint no-console: 0 */


  console.log(message);
};
var devlog = {
  log: log,
  warn: warn,
  error: error
};

// https://github.com/reduxjs/react-redux (MIT LICENSE)
// https://github.com/dai-shi/use-context-selector (MIT LICENSE)

var createUseSelector = function createUseSelector(context, subscription) {
  var defaultSelector = function defaultSelector(state) {
    return state;
  };

  function useSelector(selector) {
    var state = React.useContext(context);
    var storeSelector = selector || defaultSelector;

    var _useReducer = React.useReducer(function (s) {
      return s + 1;
    }, 0),
        _useReducer2 = _slicedToArray(_useReducer, 2),
        forceRender = _useReducer2[1];

    var selectedState;
    var latestSelector = React.useRef();
    var latestStoreState = React.useRef();
    var latestSelectedState = React.useRef();
    selectedState = latestSelectedState.current;

    try {
      if (storeSelector !== latestSelector.current || state !== latestStoreState.current) {
        selectedState = storeSelector(state);
      }
    } catch (err) {
      devlog.error(err);
    }

    useIsomorphicLayoutEffect(function () {
      latestSelector.current = storeSelector;
      latestStoreState.current = state;
      latestSelectedState.current = selectedState;
    });
    useIsomorphicLayoutEffect(function () {
      var refresh = function refresh(nextStore) {
        if (latestSelector.current === undefined || latestSelectedState.current === undefined) {
          return;
        }

        try {
          var newSelectedState = latestSelector.current(nextStore);

          if (newSelectedState === latestSelectedState.current) {
            return;
          }

          latestSelectedState.current = newSelectedState;
        } catch (err) {
          devlog.error(err);
        }

        forceRender();
      };

      subscription.add(refresh);
      return function () {
        subscription["delete"](refresh);
      };
    }, [context, subscription]);
    return selectedState;
  }

  return useSelector;
};

var createStore = function createStore(stateContext, stateSubscription, dispatchContext, dispatchSubscription) {
  var useGlobalState = createUseSelector(stateContext, stateSubscription);
  var useGlobalDispatch = createUseSelector(dispatchContext, dispatchSubscription);
  return {
    useGlobalState: useGlobalState,
    useGlobalDispatch: useGlobalDispatch
  };
};

var dispatchEventLister = function dispatchEventLister(provider, subscription) {
  var dispatcher = /*#__PURE__*/React__default['default'].memo(function (_ref) {
    var value = _ref.value,
        children = _ref.children;
    useIsomorphicLayoutEffect(function () {
      subscription.forEach(function (listener) {
        listener(value);
      });
    });
    return /*#__PURE__*/React.createElement(provider, {
      value: value
    }, children);
  });
  dispatcher.displayName = 'UseGlobalContextProvider';
  return dispatcher;
};

var createBaseContext = function createBaseContext() {
  var stateContext = /*#__PURE__*/React.createContext(null, function () {
    return 0;
  });
  var dispatchContext = /*#__PURE__*/React.createContext(null, function () {
    return 0;
  });
  var stateSubscription = new Set();
  var dispatchSubscription = new Set();
  stateContext.Provider = dispatchEventLister(stateContext.Provider, stateSubscription);
  dispatchContext.Provider = dispatchEventLister(dispatchContext.Provider, dispatchSubscription);
  stateContext.displayName = 'UseGlobalStateContext';
  dispatchContext.displayName = 'UseGlobalDispatchContext';
  return {
    state: {
      context: stateContext,
      subscription: stateSubscription
    },
    dispatch: {
      context: dispatchContext,
      subscription: dispatchSubscription
    }
  };
};

var entries = function entries(values) {
  return Object.entries(values);
};

var isFunction = function isFunction(value) {
  return value && {}.toString.call(value) === '[object Function]';
};

var createUseServerSideDispatch = function createUseServerSideDispatch(stateRef, displayName, subscription, store) {
  /* eslint no-param-reassign: 0 */
  function useServerSideDispatch(state) {
    var currentState = stateRef.current[displayName];
    var newState = isFunction(state) ? state(currentState) : state;
    stateRef.current[displayName] = newState;

    if (store) {
      store.setState(newState, displayName);
    }

    subscription.forEach(function (listener) {
      listener(stateRef.current);
    });
  }

  return useServerSideDispatch;
};

var createContext = function createContext(contextSource) {
  var _createBaseContext = createBaseContext(),
      state = _createBaseContext.state,
      dispatch = _createBaseContext.dispatch;

  var _createStore = createStore(state.context, state.subscription, dispatch.context, dispatch.subscription),
      useGlobalState = _createStore.useGlobalState,
      useGlobalDispatch = _createStore.useGlobalDispatch;

  var StateProvider = state.context.Provider;
  var DispatchProvider = dispatch.context.Provider;

  var contextProvider = function contextProvider(_ref) {
    var _store$getState;

    var children = _ref.children,
        store = _ref.store;
    var stateRef = React.useRef({});
    var dispatchRef = React.useRef({});
    var storeState = (_store$getState = store === null || store === void 0 ? void 0 : store.getState()) !== null && _store$getState !== void 0 ? _store$getState : undefined;
    entries(contextSource).forEach(function (_ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          displayName = _ref3[0],
          initialState = _ref3[1];

      var initialValue = storeState && storeState[displayName] !== undefined ? storeState[displayName] : initialState;

      var _useState = React.useState(initialValue),
          _useState2 = _slicedToArray(_useState, 2),
          hookState = _useState2[0],
          hookDispatch = _useState2[1];

      stateRef.current[displayName] = hookState;
      dispatchRef.current[displayName] = hookDispatch;

      if (store) {
        store.setState(hookState, displayName);
      }
    }, {});
    return /*#__PURE__*/React__default['default'].createElement(StateProvider, {
      value: stateRef.current
    }, /*#__PURE__*/React__default['default'].createElement(DispatchProvider, {
      value: dispatchRef.current
    }, children));
  };

  var contextServerSideProvider = function contextServerSideProvider(_ref4) {
    var _store$getState2;

    var children = _ref4.children,
        store = _ref4.store;
    var stateRef = React.useRef({});
    var dispatchRef = React.useRef({});
    var storeState = (_store$getState2 = store === null || store === void 0 ? void 0 : store.getState()) !== null && _store$getState2 !== void 0 ? _store$getState2 : undefined;
    entries(contextSource).forEach(function (_ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          displayName = _ref6[0],
          initialState = _ref6[1];

      var initialValue = storeState && storeState[displayName] !== undefined ? storeState[displayName] : initialState;
      var hookDispatch = createUseServerSideDispatch(stateRef, displayName, state.subscription, store);
      stateRef.current[displayName] = initialValue;
      dispatchRef.current[displayName] = hookDispatch;

      if (store) {
        store.setState(initialValue, displayName);
      }
    });
    return /*#__PURE__*/React__default['default'].createElement(StateProvider, {
      value: stateRef.current
    }, /*#__PURE__*/React__default['default'].createElement(DispatchProvider, {
      value: dispatchRef.current
    }, children));
  };

  return {
    useGlobalState: useGlobalState,
    useGlobalDispatch: useGlobalDispatch,
    contextProvider: isBrowser ? contextProvider : contextServerSideProvider
  };
};

/**
 * *useState* to create multiple contexts.
 * The created contexts are split into a state and a dispatch,
 * respectively, to prevent unnecessary rendering.
 */
var createUseStateContext = function createUseStateContext(contextSource) {
  var _createContext = createContext(contextSource),
      useGlobalState = _createContext.useGlobalState,
      useGlobalDispatch = _createContext.useGlobalDispatch,
      contextProvider = _createContext.contextProvider;

  return [useGlobalState, useGlobalDispatch, contextProvider];
};

var createStore$1 = function createStore(stateContext, stateSubscription, dispatchContext, dispatchSubscription) {
  var useGlobalState = createUseSelector(stateContext, stateSubscription);
  var useGlobalDispatch = createUseSelector(dispatchContext, dispatchSubscription);
  return {
    useGlobalState: useGlobalState,
    useGlobalDispatch: useGlobalDispatch
  };
};

var createUseServerSideDispatch$1 = function createUseServerSideDispatch(stateRef, displayName, reducer, subscription, store) {
  var useServerSideDispatch = function useServerSideDispatch(action) {
    /* eslint no-param-reassign: 0 */
    var currentState = stateRef.current[displayName];
    var newState = reducer.length === 1 ? reducer(currentState, undefined) : reducer(currentState, action);
    stateRef.current[displayName] = newState;

    if (store) {
      store.setState(newState, displayName);
    }

    subscription.forEach(function (listener) {
      listener(stateRef.current);
    });
  };

  return useServerSideDispatch;
};

var createContext$1 = function createContext(contextSource) {
  var _createBaseContext = createBaseContext(),
      state = _createBaseContext.state,
      dispatch = _createBaseContext.dispatch;

  var _createStore = createStore$1(state.context, state.subscription, dispatch.context, dispatch.subscription),
      useGlobalState = _createStore.useGlobalState,
      useGlobalDispatch = _createStore.useGlobalDispatch;

  var StateProvider = state.context.Provider;
  var DispatchProvider = dispatch.context.Provider;

  var contextProvider = function contextProvider(_ref) {
    var _store$getState;

    var children = _ref.children,
        store = _ref.store;
    var stateRef = React.useRef({});
    var dispatchRef = React.useRef({});
    var storeState = (_store$getState = store === null || store === void 0 ? void 0 : store.getState()) !== null && _store$getState !== void 0 ? _store$getState : undefined;
    entries(contextSource).forEach(function (_ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          displayName = _ref3[0],
          source = _ref3[1];

      var reducer = source.reducer,
          initialState = source.initialState,
          initializer = source.initializer;
      var initialValue = storeState && storeState[displayName] !== undefined ? storeState[displayName] : initialState;

      var _useReducer = React.useReducer(reducer, initialValue, initializer),
          _useReducer2 = _slicedToArray(_useReducer, 2),
          hookState = _useReducer2[0],
          hookDispatch = _useReducer2[1];

      stateRef.current[displayName] = hookState;
      dispatchRef.current[displayName] = hookDispatch;
    }, {});
    return /*#__PURE__*/React__default['default'].createElement(StateProvider, {
      value: stateRef.current
    }, /*#__PURE__*/React__default['default'].createElement(DispatchProvider, {
      value: dispatchRef.current
    }, children));
  };

  var contextServerSideProvider = function contextServerSideProvider(_ref4) {
    var _store$getState2;

    var children = _ref4.children,
        store = _ref4.store;
    var stateRef = React.useRef({});
    var dispatchRef = React.useRef({});
    var storeState = (_store$getState2 = store === null || store === void 0 ? void 0 : store.getState()) !== null && _store$getState2 !== void 0 ? _store$getState2 : undefined;
    entries(contextSource).forEach(function (_ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          displayName = _ref6[0],
          initialState = _ref6[1].initialState;

      var initialValue = storeState && storeState[displayName] !== undefined ? storeState[displayName] : initialState;
      var hookDispatch = createUseServerSideDispatch$1(stateRef, displayName, contextSource[displayName].reducer, state.subscription, store);
      stateRef.current[displayName] = initialValue;
      dispatchRef.current[displayName] = hookDispatch;
    }, {});
    return /*#__PURE__*/React__default['default'].createElement(StateProvider, {
      value: stateRef.current
    }, /*#__PURE__*/React__default['default'].createElement(DispatchProvider, {
      value: dispatchRef.current
    }, children));
  };

  return {
    useGlobalState: useGlobalState,
    useGlobalDispatch: useGlobalDispatch,
    contextProvider: isBrowser ? contextProvider : contextServerSideProvider
  };
};

/**
 * *useReducer* to create multiple contexts.
 * The created contexts are split into a state and a dispatch,
 * respectively, to prevent unnecessary rendering.
 */
var createUseReducerContext = function createUseReducerContext(contextSource) {
  var _createContext = createContext$1(contextSource),
      useGlobalState = _createContext.useGlobalState,
      useGlobalDispatch = _createContext.useGlobalDispatch,
      contextProvider = _createContext.contextProvider;

  return [useGlobalState, useGlobalDispatch, contextProvider];
};

var createInitialStore = function createInitialStore(initialState) {
  var currentState = _objectSpread2({}, initialState);

  var getState = function getState() {
    return currentState;
  };

  var setState = function setState(value, key) {
    currentState[key] = value;
  };

  var resetState = function resetState() {
    currentState = _objectSpread2({}, initialState);
  };

  return {
    getState: getState,
    setState: setState,
    resetState: resetState
  };
};

var createStore$2 = function createStore(initialValue) {
  return createInitialStore(initialValue);
};

exports.createStore = createStore$2;
exports.createUseReducerContext = createUseReducerContext;
exports.createUseStateContext = createUseStateContext;
